# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
from django.http import HttpResponse
from django.views.generic import TemplateView

from horizon import exceptions
from vsm_dashboard.api import vsm as vsmapi

LOG = logging.getLogger(__name__)

class IndexView(TemplateView):
    template_name = 'vsm/zonemgmt/index.html'

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        return context



def crushmap_datasource(request):
    # node_list = [
    #     {'type': 4,'type_name': "type4", 'id': -16, 'sg':[],'name': 'vsm','nocheck':False,"take":"",'take':"take_16"},
    #
    #     {'parent_id': [-16], 'type': 3,'type_name':"type3", 'id': -13, 'sg':[],'name': 'performances','nocheck':False,'take':"take_13"},
    #     {'parent_id': [-16], 'type': 3,'type_name':"type3", 'id': -15, 'sg':[],'name': 'capacity','nocheck':False,'take':"take_15"},
    #     {'parent_id': [-16], 'type': 3,'type_name':"type3", 'id': -14, 'sg':[],'name': 'high_performance_test','nocheck':False,'take':"take_14"},
    #
    #     {'parent_id': [-14], 'type': 2,'type_name':"type2", 'id': -11, 'sg':[],'name': 'zone0_high_performance_test','nocheck':False,'take':"take_14"},
    #     {'parent_id': [-13], 'type': 2,'type_name':"type2", 'id': -10, 'sg':[],'name': 'zone0_performance','nocheck':False,'take':"take_13"},
    #     {'parent_id': [-15], 'type': 2,'type_name':"type2", 'id': -12, 'sg':[],'name': 'zone0_capacity','nocheck':False,'take':"take_15"},
    #
    #     {'parent_id': [-10], 'type': 1,'type_name':"type1", 'id': -4, 'sg':[],'name': 'ceph02_performance_zone0','nocheck':False,'take':"take_4"},
    #     {'parent_id': [-10], 'type': 1,'type_name':"type1", 'id': -7, 'sg':[],'name': 'ceph03_performance_zone0','nocheck':False,'take':"take_7"},
    #     {'parent_id': [-10], 'type': 1,'type_name':"type1", 'id': -1, 'sg':[],'name': 'ceph01_performance_zone0','nocheck':False,'take':"take_1"},
    #     {'parent_id': [-11], 'type': 1,'type_name':"type1", 'id': -8, 'sg':[],'name': 'ceph03_high_performance_test_zone0','nocheck':False,'take':"take_8"},
    #     {'parent_id': [-11], 'type': 1,'type_name':"type1", 'id': -5, 'sg':[],'name': 'ceph02_high_performance_test_zone0','nocheck':False,'take':"take_5"},
    #     {'parent_id': [-11], 'type': 1,'type_name':"type1", 'id': -2, 'sg':[],'name': 'ceph01_high_performance_test_zone0','nocheck':False,'take':"take_2"},
    #     {'parent_id': [-12], 'type': 1,'type_name':"type1", 'id': -9, 'sg':[],'name': 'ceph03_capacity_zone0','nocheck':False,'take':"take_9"},
    #     {'parent_id': [-12], 'type': 1,'type_name':"type1", 'id': -3, 'sg':[],'name': 'ceph01_capacity_zone0','nocheck':False,'take':"take_4"},
    #     {'parent_id': [-12], 'type': 1,'type_name':"type1", 'id': -6, 'sg':[],'name': 'ceph02_capacity_zone0','nocheck':False,'take':"take_6"},
    #
    #     {'parent_id': [-1], 'type': 0,'type_name':"type0", 'id': 8, 'sg':[1,2,3],'name': 'osd.8','nocheck':True,"take":""},
    #     {'parent_id': [-1], 'type': 0,'type_name':"type0", 'id': 10, 'sg':[3],'name': 'osd.10','nocheck':True,"take":""},
    #     {'parent_id': [-1], 'type': 0,'type_name':"type0", 'id': 9, 'sg':[1,2],'name': 'osd.9','nocheck':True,"take":""},
    #     {'parent_id': [-2], 'type': 0,'type_name':"type0", 'id': 0, 'sg':[2],'name': 'osd.0','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 2, 'sg':[2],'name': 'osd.2','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 4, 'sg':[1],'name': 'osd.4','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 6, 'sg':[1],'name': 'osd.6','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 1, 'sg':[1,3],'name': 'osd.1','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 3, 'sg':[3],'name': 'osd.3','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 5, 'sg':[],'name': 'osd.5','nocheck':True,"take":""},
    #     {'parent_id': [-3], 'type': 0,'type_name':"type0", 'id': 7, 'sg':[],'name': 'osd.7','nocheck':True,"take":""},
    #     {'parent_id': [-4], 'type': 0,'type_name':"type0", 'id': 19, 'sg':[],'name': 'osd.19','nocheck':True,"take":""},
    #     {'parent_id': [-4], 'type': 0,'type_name':"type0", 'id': 20, 'sg':[],'name': 'osd.20','nocheck':True,"take":""},
    #     {'parent_id': [-4], 'type': 0,'type_name':"type0", 'id': 21, 'sg':[],'name': 'osd.21','nocheck':True,"take":""},
    #     {'parent_id': [-5], 'type': 0,'type_name':"type0", 'id': 11, 'sg':[],'name': 'osd.11','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 13, 'sg':[],'name': 'osd.13','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 12, 'sg':[],'name': 'osd.12','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 15, 'sg':[],'name': 'osd.15','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 14, 'sg':[],'name': 'osd.14','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 17, 'sg':[],'name': 'osd.17','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 16, 'sg':[],'name': 'osd.16','nocheck':True,"take":""},
    #     {'parent_id': [-6], 'type': 0,'type_name':"type0", 'id': 18, 'sg':[],'name': 'osd.18','nocheck':True,"take":""},
    #     {'parent_id': [-7], 'type': 0,'type_name':"type0", 'id': 32, 'sg':[],'name': 'osd.32','nocheck':True,"take":""},
    #     {'parent_id': [-7], 'type': 0,'type_name':"type0", 'id': 31, 'sg':[],'name': 'osd.31','nocheck':True,"take":""},
    #     {'parent_id': [-7], 'type': 0,'type_name':"type0", 'id': 30, 'sg':[],'name': 'osd.30','nocheck':True,"take":""},
    #     {'parent_id': [-8], 'type': 0,'type_name':"type0", 'id': 22, 'sg':[],'name': 'osd.22','nocheck':True,"take":""},
    #     {'parent_id': [-9], 'type': 0,'type_name':"type0", 'id': 25, 'sg':[],'name': 'osd.25','nocheck':True,"take":""},
    #     {'parent_id': [-9], 'type': 0,'type_name':"type0", 'id': 26, 'sg':[],'name': 'osd.26','nocheck':True,"take":""},
    #     {'parent_id': [-9], 'type': 0,'type_name':"type0", 'id': 27, 'sg':[],'name': 'osd.27','nocheck':True,"take":""},
    #     {'parent_id': [-9], 'type': 0,'type_name':"type0", 'id': 23, 'sg':[],'name': 'osd.23','nocheck':True,"take":""},
    #     {'parent_id': [-9], 'type': 0,'type_name':"type0", 'id': 28, 'sg':[],'name': 'osd.28','nocheck':True,"take":""},
    #     {'parent_id': [-9], 'type': 0,'type_name':"type0", 'id': 29, 'sg':[],'name': 'osd.29','nocheck':True,"take":""},
    #     {'parent_id': [-9,-8], 'type': 0,'type_name':"type0", 'id': 24, 'sg':[],'name': 'osd.24','nocheck':True,"take":""},
    # ]
    body = {'cluster_id':1}
    ret_code,node_list = vsmapi.get_crushmap_tree_data(request,body)
    node_list = node_list['tree_node']
    print 'node_list=======',node_list

    #generate the crushmap nodes
    crushmap_nodes = []
    type_list = []
    for node in node_list:
    	#ignore the osd node 
    	if node["type"] == 0:
    		continue;

        is_parent = False
        is_open = False
        font = ""

        #get the type list
        is_type_exist = False
        for type_item in type_list:
            if type_item["id"] == node["type"]:
                is_type_exist = True
                break
        if is_type_exist == False:
            type_list.append({
                "id":node["type"],
                "name":node["type_name"],
            })

        #if parent id is none,it's "root"
        pID_list = node.get("parent_id",["root"])
        #print '111===',node
        for pID in pID_list:
            if pID == "root":
                is_parent = True
                is_open = True
                pID = "root"
            item = {
                "id":node["id"],
                "pId":pID,
                "type_id":node["type"],
                "type_name":node["type_name"],
                "name":node["name"],
                "font":"",
                "open":is_open,
                "isParent":is_parent,
            }
            #append the item into crushmap
            #print '33333333333'
            crushmap_nodes.append(item)
    #sort the type_list
    #print '222===',type_list
    type_list.sort(key=lambda x:x["id"])
    resp = dict(message="", status="OK",crushmap=crushmap_nodes,type_list=type_list)
    resp = json.dumps(resp)
    return HttpResponse(resp)


def add(request):
    body = json.loads(request.body)
    try:
        #TODO: add zone....
        code,ret = vsmapi.add_zone_to_crushmap_and_db(request,body)

        status = "OK"
        msg = "Successfully!"
    except ex:
        print ex
        status = "Failed"
        msg = "Add Zone Failed!Unknown exception!"

    resp = dict(message=msg, status=status)
    resp = json.dumps(resp)
    return HttpResponse(resp)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy

from horizon import exceptions
from horizon import tables
from horizon import forms
from horizon import views

from vsm_dashboard.api import vsm as vsmapi
from .tables import ListMonitorStatusTable
from django.http import HttpResponse
from vsm_dashboard.common.horizon.summary import SummaryRenderer

import json
LOG = logging.getLogger(__name__)

from django.utils.datastructures import SortedDict
from vsm_dashboard.utils import get_time_delta

class ModalSummaryMixin(object):

    def get_context_data(self, **kwargs):
        context = super(ModalSummaryMixin, self).get_context_data(**kwargs)
        return context

class IndexView(ModalSummaryMixin, tables.DataTableView):
    table_class = ListMonitorStatusTable
    template_name = 'vsm/monitor-status/index.html'

    def get_data(self):
        _monitor_status = []
        #_monitors= vsmapi.get_monitor_list(self.request,)
        try:
            _monitor_status = vsmapi.monitor_status(self.request,)
            LOG.info("MONITOR STATUS: %s "%_monitor_status)
            #LOG.error(_monitor_status)
            #LOG.error("")
            if _monitor_status:
                logging.debug("resp body in view: %s" % _monitor_status)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve mon list. '))

        monitor_status = []
        for _monitor in _monitor_status:
            monitor = {"id": _monitor.id,
                        "name": _monitor.name,
                        "address": _monitor.address,
                        "health": _monitor.health,
                        "details": _monitor.details,
                        "skew": _monitor.skew,
                        "latency": _monitor.latency,
                        "mb_total": 0 if not _monitor.kb_total else int(_monitor.kb_total/1024),
                        "mb_used": 0 if not _monitor.kb_used else int(_monitor.kb_used/1024),
                        "mb_avail": 0 if not _monitor.kb_avail else int(_monitor.kb_avail/1024),
                        "percent_avail": _monitor.avail_percent,
                        "updated_at": get_time_delta(_monitor.updated_at),
                      }

            monitor_status.append(monitor)
        return monitor_status




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon import exceptions
from horizon import messages
from vsm_dashboard.api import vsm as vsmapi

LOG = logging.getLogger(__name__)

class AddECProfileAction(tables.LinkAction):
    name = "add_ec_profile"
    verbose_name = _("New")
    url = "/dashboard/vsm/ecprofiles-management/add_ec_profile_view/"
    classes = ('btn-primary',)

class RemoveECProfilesAction(tables.LinkAction):
    name = "remove_ec_profiles"
    verbose_name = _("Remove")
    classes = ('btn-primary',)
    url = "horizon:vsm:ecprofiles-management:index"



class ECProfilesTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("ID"), hidden=True)
    name = tables.Column("name", verbose_name=_("Name"))
    plugin = tables.Column("plugin", verbose_name=_("Plugin Name"))
    plugin_path = tables.Column("plugin_path", verbose_name=_("Plugin Path"))
    pg_num = tables.Column("pg_num", verbose_name=_("PG Number"))
    k_v_dict = tables.Column("plugin_kv_pair", verbose_name=_("Key Values Pairs"))
    class Meta:
        name = "ec_profiles"
        verbose_name = _("EC Profiles List")
        table_actions = (AddECProfileAction,RemoveECProfilesAction)

    def get_object_id(self, datum):
        if hasattr(datum, "id"):
            return datum.id
        else:
            return datum["id"]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _
from horizon import tables
STRING_SEPARATOR = "__"

STATUS_DISPLAY_CHOICES = (
    ("resize", "Resize/Migrate"),
    ("verify_resize", "Confirm or Revert Resize/Migrate"),
    ("revert_resize", "Revert Resize/Migrate"),
)

class ImportClusterAction(tables.LinkAction):
    name = "import_cluster"
    verbose_name = _("Import Cluster")
    url = "horizon:vsm:cluster-import:importcluster"
    classes = ("ajax-modal", "btn-primary")

    

class ListServerTable(tables.DataTable):
    STATUS_CHOICES = (
        ("active", True),
        ("available", True),
        ("Active", True),
    )

    server_id = tables.Column("id", verbose_name=_("ID"))
    name = tables.Column("name", verbose_name=_("Name"))
    primary_public_ip = tables.Column("primary_public_ip", verbose_name=_("Management Address"), attrs={"data-type": "ip"})
    secondary_public_ip = tables.Column("secondary_public_ip", verbose_name=_("Public Address"), attrs={"data-type": "ip"})
    cluster_ip = tables.Column("cluster_ip", verbose_name=_("Cluster Address"), attrs={"data-type": "ip"})
    osds = tables.Column("osds", verbose_name=_("OSDs"))
    is_monitor = tables.Column("is_monitor", verbose_name=_("Monitor"))
    zone = tables.Column("zone", classes=("zone",), verbose_name=_("Zone"))
    status = tables.Column("status", status=True,
                            status_choices=STATUS_CHOICES,
                            display_choices=STATUS_DISPLAY_CHOICES,
                            verbose_name=_("Status"))

    class Meta:
        name = "server_list"
        verbose_name = _("Server List")
        table_actions = (ImportClusterAction,)
        status_columns = ['status']
        multi_select = False

    def get_object_id(self, datum):
        if hasattr(datum, "id"):
            return datum.id
        else:
            return datum["id"]

    def get_object_display(self, datum):
        if hasattr(datum, "name"):
            return datum.id
        else:
            return datum["name"]





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django import template
from django.views.generic import View
import json

class ChartRenderer(View):
    def __init__(self, request):
        self.request = request

    def _get_chart(self):
        return json.dumps(self.get_chart())

    def get_chart(self):
        raise NotImplementedError

    def render(self):
        summary_template = template.loader.get_template("vsm/chart.html")
        context = template.Context({"name": self.name, "datas": self._get_chart()})
        return summary_template.render(context)




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy

from horizon import exceptions
from horizon import tables
from horizon import forms
from horizon import views

from vsm_dashboard.api import vsm as vsmapi
from .tables import ListMDSStatusTable
from django.http import HttpResponse

import json
LOG = logging.getLogger(__name__)

from django.utils.datastructures import SortedDict

from vsm_dashboard.utils import get_time_delta

class ModalSummaryMixin(object):

   def get_context_data(self, **kwargs):
        context = super(ModalSummaryMixin, self).get_context_data(**kwargs)
        return context

class IndexView(ModalSummaryMixin, tables.DataTableView):
    table_class = ListMDSStatusTable
    template_name = 'vsm/mds-status/index.html'

    def get_data(self):
        _servers = []
        #_servers= vsmapi.get_server_list(self.request,)
        _mds_status = []
        try:
            _mds_status = vsmapi.mds_status(self.request)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve sever list. '))

        mds_status = []
        for _mds in _mds_status:
            mds = {"gid": _mds.gid,
                   "id": _mds.id,
                        "name": _mds.name,
                        "state": _mds.state,
                        "address": _mds.address,
                        "updated_at": get_time_delta(_mds.updated_at),
                        }
            mds_status.append(mds)
        return mds_status





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from .mixin import BatchActionMixin
from horizon.tables import BatchAction

import logging
from horizon import exceptions
from horizon import messages
from horizon.utils import functions
from django.utils.translation import ugettext_lazy as _
from django import shortcuts
LOG = logging.getLogger(__name__)

class BatchAction(BatchAction):
    def handle(self, table, request, obj_ids):
        action_success = []
        action_failure = []
        action_not_allowed = []
        for datum_id in obj_ids:
            datum = table.get_object_by_id(datum_id)
            datum_display = table.get_object_display(datum) or _("N/A")
            if not table._filter_action(self, request, datum):
                action_not_allowed.append(datum_display)
                LOG.info('Permission denied to %s: "%s"' %
                         (self._get_action_name(past=True).lower(),
                          datum_display))
                continue
            try:
                self.action(request, datum_id)
                #Call update to invoke changes if needed
                self.update(request, datum)
                action_success.append(datum_display)
                self.success_ids.append(datum_id)
                LOG.info('%s: "%s"' %
                         (self._get_action_name(past=True), datum_display))
            except Exception as ex:
                # Handle the exception but silence it since we'll display
                # an aggregate error message later. Otherwise we'd get
                # multiple error messages displayed to the user.
                if getattr(ex, "_safe_message", None):
                    ignore = False
                else:
                    ignore = True
                    action_failure.append(datum_display)
                exceptions.handle(request, ignore=ignore)

        # Begin with success message class, downgrade to info if problems.
        success_message_level = messages.success
        if action_not_allowed:
            msg = 'You are not allowed to %(action)s: %(objs)s'
            params = {"action":
                      self._get_action_name(action_not_allowed).lower(),
                      "objs": functions.lazy_join(", ", action_not_allowed)}
            messages.error(request, msg % params)
            success_message_level = messages.info
        if action_failure:
            msg = _('Unable to %(action)s: %(objs)s')
            params = {"action": self._get_action_name(action_failure).lower(),
                      "objs": functions.lazy_join(", ", action_failure)}
            messages.error(request, msg % params)
            success_message_level = messages.info
        if action_success:
            msg = _('%(action)s: %(objs)s')
            params = {"action":
                      self._get_action_name(action_success, past=True),
                      "objs": functions.lazy_join(", ", action_success)}
            success_message_level(request, msg % params)

        return shortcuts.redirect(self.get_success_url(request))




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import division

from calendar import monthrange
import datetime
import logging

from django.utils.translation import ugettext as _
from django.utils import timezone

from horizon import exceptions
from horizon import forms
from horizon import messages

from vsm_dashboard import api
from vsm_dashboard.usage import quotas

LOG = logging.getLogger(__name__)

def almost_now(input_time):
    now = timezone.make_naive(timezone.now(), timezone.utc)
    # If we're less than a minute apart we'll assume success here.
    return now - input_time < datetime.timedelta(seconds=30)

class BaseUsage(object):
    show_terminated = False

    def __init__(self, request, tenant_id=None):
        self.tenant_id = tenant_id or request.user.tenant_id
        self.request = request
        self.summary = {}
        self.usage_list = []
        self.quotas = {}

    @property
    def today(self):
        return timezone.now()

    @staticmethod
    def get_start(year, month, day=1):
        start = datetime.datetime(year, month, day, 0, 0, 0)
        return timezone.make_aware(start, timezone.utc)

    @staticmethod
    def get_end(year, month, day=1):
        days_in_month = monthrange(year, month)[1]
        period = datetime.timedelta(days=days_in_month)
        end = BaseUsage.get_start(year, month, day) + period
        # End our calculation at midnight of the given day.
        date_end = datetime.datetime.combine(end, datetime.time(0, 0, 0))
        date_end = timezone.make_aware(date_end, timezone.utc)
        if date_end > timezone.now():
            date_end = timezone.now()
        return date_end

    def get_instances(self):
        instance_list = []
        [instance_list.extend(u.server_usages) for u in self.usage_list]
        return instance_list

    def get_date_range(self):
        if not hasattr(self, "start") or not hasattr(self, "end"):
            args = (self.today.year, self.today.month)
            form = self.get_form()
            if form.is_valid():
                args = (int(form.cleaned_data['year']),
                        int(form.cleaned_data['month']))
            self.start = self.get_start(*args)
            self.end = self.get_end(*args)
        return self.start, self.end

    def get_form(self):
        if not hasattr(self, 'form'):
            if any(key in ['month', 'year'] for key in self.request.GET):
                # bound form
                self.form = forms.DateForm(self.request.GET)
            else:
                # non-bound form
                self.form = forms.DateForm(initial={'month': self.today.month,
                                                    'year': self.today.year})
        return self.form

    def get_usage_list(self, start, end):
        raise NotImplementedError("You must define a get_usage method.")

    def summarize(self, start, end):
        if start <= end <= self.today:
            # The API can't handle timezone aware datetime, so convert back
            # to naive UTC just for this last step.
            start = timezone.make_naive(start, timezone.utc)
            end = timezone.make_naive(end, timezone.utc)
            try:
                self.usage_list = self.get_usage_list(start, end)
            except:
                exceptions.handle(self.request,
                                  _('Unable to retrieve usage information.'))
        else:
            messages.info(self.request,
                          _("You are viewing data for the future, "
                            "which may or may not exist."))

        for tenant_usage in self.usage_list:
            tenant_summary = tenant_usage.get_summary()
            for key, value in tenant_summary.items():
                self.summary.setdefault(key, 0)
                self.summary[key] += value

    def get_quotas(self):
        try:
            self.quotas = quotas.tenant_quota_usages(self.request)
        except:
            exceptions.handle(self.request,
                              _("Unable to retrieve quota information."))

    def csv_link(self):
        form = self.get_form()
        if hasattr(form, "cleaned_data"):
            data = form.cleaned_data
        else:
            data = {"month": self.today.month, "year": self.today.year}
        return "?month=%s&year=%s&format=csv" % (data['month'], data['year'])

class GlobalUsage(BaseUsage):
    show_terminated = True

    def get_usage_list(self, start, end):
        return api.nova.usage_list(self.request, start, end)

class TenantUsage(BaseUsage):
    attrs = ('memory_mb', 'vcpus', 'uptime',
             'hours', 'local_gb')

    def get_usage_list(self, start, end):
        show_terminated = self.request.GET.get('show_terminated',
                                               self.show_terminated)
        instances = []
        terminated_instances = []
        usage = api.nova.usage_get(self.request, self.tenant_id, start, end)
        # Attribute may not exist if there are no instances
        if hasattr(usage, 'server_usages'):
            now = self.today
            for server_usage in usage.server_usages:
                # This is a way to phrase uptime in a way that is compatible
                # with the 'timesince' filter. (Use of local time intentional.)
                server_uptime = server_usage['uptime']
                total_uptime = now - datetime.timedelta(seconds=server_uptime)
                server_usage['uptime_at'] = total_uptime
                if server_usage['ended_at'] and not show_terminated:
                    terminated_instances.append(server_usage)
                else:
                    instances.append(server_usage)
        usage.server_usages = instances
        return (usage,)
